==> trip-0.1.0.tar.gz: trip-0.1.0/util/vcondition-vxworks.cxx <==

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const vcondition_vxworks_cxx_Version =
    "$Id: vcondition-vxworks.cxx,v 1.10 2000/09/20 08:01:35 cvs Exp $";


#ifdef __vxworks

#include <cassert>
#include "VMutex.h"
#include "VCondition.h"
#include <stdio.h>

/*********************************************************************
 
VCondition for Linux
 
*********************************************************************/

VCondition::VCondition (void)
{
    // conditional variable is implemented as a binary semaphore
    // which is initialized to empty
    myId = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    assert(myId);
}


VCondition::~VCondition (void)
{
    STATUS errorcode = semDelete(myId);
    assert(errorcode == OK);
}


int
VCondition::wait (VMutex* mutex, int relativeTimeInMs)
{
    int retval;

    int delayInTicks = WAIT_FOREVER;

    if ( relativeTimeInMs > -1 )
    {
        int ticksPerSecond = sysClkRateGet();
        assert(ticksPerSecond > 0);
        delayInTicks = (relativeTimeInMs * 1000) * ticksPerSecond;
    }

    // note, the call to taskLock() guarantees that the task will not
    // be scheduled out; however, it will still give up the cpu if it
    // blocks on a semaphore, thereby avoiding a deadlock
    assert(taskLock() == OK);
    assert(mutex->unlock() == 0);
    retval = (int) semTake(myId, delayInTicks);

    if ( retval != OK )
    {
        assert(errno == S_objLib_OBJ_TIMEOUT);
    }

    assert(mutex->lock() == 0);
    assert(taskUnlock() == OK);
    return retval;
}


int
VCondition::signal (void)
{
    return ( (int) semGive(myId) );
}


const vcondition_t*
VCondition::getId (void)
{
    return &myId;
}

#endif // __vxworks

==> trip-1.0.0.tar.gz: trip-1.0.0/util/vcondition-vxworks.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const vcondition_vxworks_cxx_Version =
    "$Id: vcondition-vxworks.cxx,v 1.14 2001/03/09 08:30:46 icahoon Exp $";

#ifdef __vxworks

#include <cassert>
#include "VMutex.h"
#include "VCondition.h"
#include <stdio.h>

/*********************************************************************
 
VCondition for VxWorks
 
*********************************************************************/

VCondition::VCondition ()
{
    // conditional variable is implemented as a binary semaphore
    // which is initialized to empty
    myId = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    assert(myId);
}


VCondition::~VCondition ()
{
    STATUS errorcode = semDelete(myId);
    assert(errorcode == OK);
}


int
VCondition::wait (VMutex* mutex, int relativeTimeInMs)
{
    int utime = relativeTimeInMs < 1
                    ? relativeTimeInMs * 1000
                    : relativeTimeInMs;
    
    return ( uwait(mutex, utime) );
}


int
VCondition::uwait (VMutex* mutex, int relativeTimeInUs)
{
    int retval;

    int delayInTicks = WAIT_FOREVER;

    if ( relativeTimeInUs > -1 )
    {
        int ticksPerSecond = sysClkRateGet();
        assert(ticksPerSecond > 0);
        delayInTicks = relativeTimeInUs * ( ticksPerSecond / 1000000 );
    }

    // note, the call to taskLock() guarantees that the task will not
    // be scheduled out; however, it will still give up the cpu if it
    // blocks on a semaphore, thereby avoiding a deadlock
    bool taskLockCode = taskLock();
    assert(taskLockCode == OK);
    bool unlockCode = mutex->unlock();
    assert(unlockCode == 0);
    retval = (int) semTake(myId, delayInTicks);

    if ( retval != OK )
    {
        assert(errno == S_objLib_OBJ_TIMEOUT);
    }

    bool mutexLockCode = mutex->lock();
    assert(mutexLockCode == 0);
    bool taskUnlockCode = taskUnlock();
    assert(taskUnlockCode == OK);
    return retval;
}


int
VCondition::signal ()
{
    return ( (int) semGive(myId) );
}


int
VCondition::broadcast()
{
    // TODO
    assert( 0 );
    return ( (int) semGive(myId) );
}


const vcondition_t*
VCondition::getId () const
{
    return &myId;
}

// __vxworks
#endif
